@@ -1,5 +1,7 @@
 import os
 import io
+import json
+import datetime
 import tempfile
 import hashlib
 import requests
@@ -32,6 +34,14 @@ class DatameshWriteError(Exception):
     pass
 
 
+def json_serial(obj):
+    """JSON serializer for objects not serializable by default json code"""
+
+    if isinstance(obj, (datetime.datetime, datetime.date)):
+        return obj.isoformat()
+    raise TypeError("Type %s not serializable" % type(obj))
+
+
 def asyncwrapper(func):
     @wraps(func)
     async def run(*args, loop=None, executor=None, **kwargs):
@@ -136,11 +146,14 @@ class Connector(object):
             raise DatameshConnectError(resp.text)
         return True
 
-    def _zarr_proxy(self, datasource_id):
+    def _zarr_proxy(self, datasource_id, parameters={}):
         try:
             mapper = fsspec.get_mapper(
                 f"{self._gateway}/zarr/{datasource_id}",
-                headers=self._auth_headers,
+                headers={
+                    **self._auth_headers,
+                    "X-PARAMETERS": json.dumps(parameters, default=json_serial),
+                },
             )
         except Exception as e:
             raise DatameshConnectError(str(e))
@@ -230,6 +243,8 @@ class Connector(object):
                 with tempfile.NamedTemporaryFile("wb") as f:
                     f.write(resp.content)
                     f.seek(0)
+                    if stage.container == Container.Dataset:
+                        return xarray.load_dataset(f.name)
                     if stage.container == Container.GeoDataFrame:
                         return geopandas.read_parquet(f.name)
                     else:
@@ -282,7 +297,9 @@ class Connector(object):
             "geom": meta_dict["geometry"],
             **meta_dict["properties"],
         }
-        return Datasource(**props)
+        ds = Datasource(**props)
+        ds._exists = True
+        return ds
 
     @asyncwrapper
     def get_datasource_async(self, datasource_id):
@@ -301,24 +318,27 @@ class Connector(object):
         """
         return self.get_datasource(datasource_id)
 
-    def load_datasource(self, datasource_id, use_dask=True):
+    def load_datasource(self, datasource_id, parameters={}, use_dask=True):
         """Load a datasource into the work environment.
         For datasources which load into DataFrames or GeoDataFrames, this returns an in memory instance of the DataFrame.
         For datasources which load into an xarray Dataset, an open zarr backed dataset is returned.
 
         Args:
             datasource_id (string): Unique datasource id
+            parameters (dict): Additional datasource parameters
             use_dask (bool, optional): Load datasource as a dask enabled datasource if possible. Defaults to True.
 
         Returns:
             Union[:obj:`pandas.DataFrame`, :obj:`geopandas.GeoDataFrame`, :obj:`xarray.Dataset`]: The datasource container
         """
-        stage = self._stage_request(Query(datasource=datasource_id))
+        stage = self._stage_request(
+            Query(datasource=datasource_id, parameters=parameters)
+        )
         if stage is None:
             warnings.warn("No data found for query")
             return None
         if stage.container == Container.Dataset:
-            mapper = self._zarr_proxy(datasource_id)
+            mapper = self._zarr_proxy(datasource_id, parameters)
             return xarray.open_zarr(
                 mapper, consolidated=True, decode_coords="all", mask_and_scale=True
             )
@@ -355,7 +375,7 @@ class Connector(object):
         Returns:
             Union[:obj:`pandas.DataFrame`, :obj:`geopandas.GeoDataFrame`, :obj:`xarray.Dataset`]: The datasource container
         """
-        return self._query(query)
+        return self._query(query, use_dask)
 
     @asyncwrapper
     async def query_async(self, query, use_dask=True):
@@ -385,7 +405,7 @@ class Connector(object):
 
         Args:
             datasource_id (string): Unique datasource id
-            data (Union[:obj:`pandas.DataFrame`, :obj:`geopandas.GeoDataFrame`, :obj:`xarray.Dataset`]): The data to be written to datamesh
+            data (Union[:obj:`pandas.DataFrame`, :obj:`geopandas.GeoDataFrame`, :obj:`xarray.Dataset`, None]):  The data to be written to datamesh. If data is None, just update metadata properties.
             geometry (:obj:`oceanum.datasource.Geometry`, optional): GeoJSON geometry of the datasource
             append (string, optional): Coordinate to append on. default=None
             overwrite (bool, optional): Overwrite existing datasource. default=False
@@ -398,44 +418,45 @@ class Connector(object):
             ds = self.get_datasource(datasource_id)
         except DatameshConnectError as e:
             overwrite = True
-        with tempfile.NamedTemporaryFile("w+b", delete=False) as f:
-            try:
-                if isinstance(data, xarray.Dataset):
-                    data.to_netcdf(f.name)
-                    f.seek(0)
-                    ds = self._data_write(
-                        datasource_id,
-                        f.read(),
-                        "application/x-netcdf4",
-                        append,
-                        overwrite,
-                    )
-                else:
-                    data.to_parquet(f, index=True)
-                    f.seek(0)
-                    ds = self._data_write(
-                        datasource_id,
-                        f.read(),
-                        "application/parquet",
-                        append,
-                        overwrite,
-                    )
-                ds._exists = True
-            except Exception as e:
-                raise DatameshWriteError(e)
-            finally:
-                os.remove(f.name)
-
-            for key in properties:
+        if data is not None:
+            with tempfile.NamedTemporaryFile("w+b", delete=False) as f:
+                try:
+                    if isinstance(data, xarray.Dataset):
+                        data.to_netcdf(f.name)
+                        f.seek(0)
+                        ds = self._data_write(
+                            datasource_id,
+                            f.read(),
+                            "application/x-netcdf4",
+                            append,
+                            overwrite,
+                        )
+                    else:
+                        data.to_parquet(f, index=True)
+                        f.seek(0)
+                        ds = self._data_write(
+                            datasource_id,
+                            f.read(),
+                            "application/parquet",
+                            append,
+                            overwrite,
+                        )
+                    ds._exists = True
+                except Exception as e:
+                    raise DatameshWriteError(e)
+                finally:
+                    os.remove(f.name)
+        elif overwrite:
+            ds = Datasource(id=datasource_id, geom=geometry, **properties)
+        for key in properties:
+            if key not in ["driver", "schema"]:
                 setattr(ds, key, properties[key])
-            if geometry:
-                ds.geom = geometry
-            try:
-                self._metadata_write(ds)
-            except:
-                raise DatameshWriteError(
-                    "Cannot register datasource {datasource_id}: {e}"
-                )
+        if geometry:
+            ds.geom = geometry
+        try:
+            self._metadata_write(ds)
+        except:
+            raise DatameshWriteError("Cannot register datasource {datasource_id}: {e}")
         return ds
 
     @asyncwrapper
@@ -446,7 +467,7 @@ class Connector(object):
 
         Args:
             datasource_id (string): Unique datasource id
-            data (Union[:obj:`pandas.DataFrame`, :obj:`geopandas.GeoDataFrame`, :obj:`xarray.Dataset`]): The data to be written to datamesh
+            data (Union[:obj:`pandas.DataFrame`, :obj:`geopandas.GeoDataFrame`, :obj:`xarray.Dataset`, None]): The data to be written to datamesh. If data is None, just update metadata properties.
             geometry (:obj:`oceanum.datasource.Geometry`): GeoJSON geometry of the datasource
             append (string, optional): Coordinate to append on. default=None
             overwrite (bool, optional): Overwrite existing datasource. default=False
